import type { AppLoadContext } from "./server-runtime/data";
import type { Serializable } from "./server-runtime/single-fetch";
export type Expect<T extends true> = T;
type Equal<X, Y> = (<T>() => T extends X ? 1 : 2) extends (<T>() => T extends Y ? 1 : 2) ? true : false;
type IsAny<T> = 0 extends 1 & T ? true : false;
type IsDefined<T> = Equal<T, undefined> extends true ? false : true;
type Fn = (...args: any[]) => unknown;
type RouteModule = {
    loader?: Fn;
    clientLoader?: Fn;
    action?: Fn;
    clientAction?: Fn;
    HydrateFallback?: unknown;
    default?: unknown;
    ErrorBoundary?: unknown;
};
type VoidToUndefined<T> = Equal<T, void> extends true ? undefined : T;
type DataFrom<T> = IsAny<T> extends true ? undefined : T extends Fn ? VoidToUndefined<Awaited<ReturnType<T>>> : undefined;
type ServerDataFrom<T> = Serialize<DataFrom<T>>;
type ClientDataFrom<T> = DataFrom<T>;
export type LoaderData<T extends RouteModule> = _LoaderData<ServerDataFrom<T["loader"]>, ClientDataFrom<T["clientLoader"]>, false, // TODO
IsAny<T["HydrateFallback"]> extends true ? false : true>;
type _LoaderData<ServerLoaderData, ClientLoaderData, ClientLoaderHydrate extends boolean, HasHydrateFallback> = [
    HasHydrateFallback,
    ClientLoaderHydrate
] extends [true, true] ? IsDefined<ClientLoaderData> extends true ? ClientLoaderData : undefined : [
    IsDefined<ClientLoaderData>,
    IsDefined<ServerLoaderData>
] extends [true, true] ? ServerLoaderData | ClientLoaderData : IsDefined<ClientLoaderData> extends true ? ClientLoaderHydrate extends true ? ClientLoaderData : ClientLoaderData | undefined : IsDefined<ServerLoaderData> extends true ? ServerLoaderData : undefined;
export type ActionData<T extends RouteModule> = _ActionData<ServerDataFrom<T["action"]>, ClientDataFrom<T["clientAction"]>>;
type _ActionData<ServerActionData, ClientActionData> = Awaited<[
    IsDefined<ServerActionData>,
    IsDefined<ClientActionData>
] extends [true, true] ? ServerActionData | ClientActionData : IsDefined<ClientActionData> extends true ? ClientActionData : IsDefined<ServerActionData> extends true ? ServerActionData : undefined>;
type DataFunctionArgs<Params> = {
    request: Request;
    params: Params;
    context?: AppLoadContext;
};
type Serialize<T> = T extends Serializable ? T : T extends (...args: any[]) => unknown ? undefined : T extends Promise<infer U> ? Promise<Serialize<U>> : T extends Map<infer K, infer V> ? Map<Serialize<K>, Serialize<V>> : T extends Set<infer U> ? Set<Serialize<U>> : T extends [] ? [] : T extends readonly [infer F, ...infer R] ? [Serialize<F>, ...Serialize<R>] : T extends Array<infer U> ? Array<Serialize<U>> : T extends readonly unknown[] ? readonly Serialize<T[number]>[] : T extends Record<any, any> ? {
    [K in keyof T]: Serialize<T[K]>;
} : undefined;
export type ServerLoaderArgs<Params> = DataFunctionArgs<Params>;
export type ClientLoaderArgs<Params, T extends RouteModule> = DataFunctionArgs<Params> & {
    serverLoader: () => Promise<ServerDataFrom<T["loader"]>>;
};
export type ServerActionArgs<Params> = DataFunctionArgs<Params>;
export type ClientActionArgs<Params, T extends RouteModule> = DataFunctionArgs<Params> & {
    serverAction: () => Promise<ServerDataFrom<T["action"]>>;
};
export type HydrateFallbackProps<Params> = {
    params: Params;
};
export type DefaultProps<Params, LoaderData, ActionData> = {
    params: Params;
    loaderData: LoaderData;
    actionData?: ActionData;
};
export type ErrorBoundaryProps<Params, LoaderData, ActionData> = {
    params: Params;
    error: unknown;
    loaderData?: LoaderData;
    actionData?: ActionData;
};
export {};
